@@ -39,15 +39,18 @@
 #include "DisallowMacroScratchRegisterUsage.h"
 #include "Reg.h"
 #include <wtf/ListDump.h>
+#include <wtf/SequesteredMalloc.h>
 #include <wtf/TZoneMallocInlines.h>
 
+WTF_ALLOW_UNSAFE_BUFFER_USAGE_BEGIN
+
 namespace JSC { namespace B3 { namespace Air {
 
 namespace GenerateAndAllocateRegistersInternal {
-static bool verbose = false;
+static constexpr bool verbose = false;
 }
 
-WTF_MAKE_TZONE_ALLOCATED_IMPL(GenerateAndAllocateRegisters);
+WTF_MAKE_SEQUESTERED_ARENA_ALLOCATED_IMPL(GenerateAndAllocateRegisters);
 
 GenerateAndAllocateRegisters::GenerateAndAllocateRegisters(Code& code)
     : m_code(code)
@@ -117,7 +120,7 @@ void GenerateAndAllocateRegisters::insertBlocksForFlushAfterTerminalPatchpoints(
         if (inst.kind.opcode != Patch)
             continue;
 
-        HashMap<Tmp, Arg*> needToDef;
+        UncheckedKeyHashMap<Tmp, Arg*> needToDef;
 
         inst.forEachArg([&] (Arg& arg, Arg::Role role, Bank, Width) {
             if (!arg.isTmp())
@@ -396,7 +399,7 @@ void GenerateAndAllocateRegisters::prepareForGeneration()
     });
 #endif
 
-    m_liveness = makeUnique<UnifiedTmpLiveness>(m_code);
+    m_liveness = makeUniqueWithoutFastMallocCheck<UnifiedTmpLiveness>(m_code);
 
     {
         buildLiveRanges(*m_liveness);
@@ -505,8 +508,8 @@ void GenerateAndAllocateRegisters::prepareForGeneration()
         UnifiedTmpLiveness liveness(m_code);
         for (BasicBlock* block : m_code) {
             auto assertLivenessAreEqual = [&] (auto a, auto b) {
-                HashSet<Tmp> livenessA;
-                HashSet<Tmp> livenessB;
+                UncheckedKeyHashSet<Tmp> livenessA;
+                UncheckedKeyHashSet<Tmp> livenessB;
                 for (Tmp tmp : a) {
                     if (tmp.isReg() && isDisallowedRegister(tmp.reg()))
                         continue;
@@ -580,6 +583,16 @@ void GenerateAndAllocateRegisters::generate(CCallHelpers& jit)
     };
 
     Disassembler* disassembler = m_code.disassembler();
+    PCToOriginMap& pcToOriginMap = m_code.proc().pcToOriginMap();
+    auto addItem = [&](Inst& inst) {
+        if (!m_code.shouldPreserveB3Origins())
+            return;
+        if (inst.origin)
+            pcToOriginMap.appendItem(m_jit->labelIgnoringWatchpoints(), inst.origin->origin());
+        else
+            pcToOriginMap.appendItem(m_jit->labelIgnoringWatchpoints(), Origin());
+    };
+
 
     m_globalInstIndex = 1;
 
@@ -908,8 +921,11 @@ void GenerateAndAllocateRegisters::generate(CCallHelpers& jit)
 
             if (!inst.isTerminal()) {
                 CCallHelpers::Jump jump;
-                if (needsToGenerate)
+                if (needsToGenerate) {
+                    // Register allocation is done, emit PC label to origin map before emitting bytes.
+                    addItem(inst);
                     jump = inst.generate(*m_jit, context);
+                }
                 ASSERT_UNUSED(jump, !jump.isSet());
 
                 handleClobber();
@@ -952,10 +968,12 @@ void GenerateAndAllocateRegisters::generate(CCallHelpers& jit)
 
                     // We currently don't represent the full epilogue in Air, so we need to
                     // have this override.
+                    addItem(inst);
                     m_code.emitEpilogue(*m_jit);
                 }
                 
                 if (needsToGenerate) {
+                    addItem(inst);
                     CCallHelpers::Jump jump = inst.generate(*m_jit, context);
 
                     // The jump won't be set for patchpoints. It won't be set for Oops because then it won't have
@@ -1000,7 +1018,7 @@ void GenerateAndAllocateRegisters::generate(CCallHelpers& jit)
 
     for (auto& entry : m_blocksAfterTerminalPatchForSpilling) {
         entry.value.jump.linkTo(m_jit->label(), m_jit);
-        const HashMap<Tmp, Arg*>& spills = entry.value.defdTmps;
+        const UncheckedKeyHashMap<Tmp, Arg*>& spills = entry.value.defdTmps;
         for (auto& entry : spills) {
             Arg* arg = entry.value;
             if (!arg->isTmp())
@@ -1021,17 +1039,23 @@ void GenerateAndAllocateRegisters::generate(CCallHelpers& jit)
         entrypointLabels[i] = *context.blockLabels[m_code.entrypoint(i).block()];
     m_code.setEntrypointLabels(WTFMove(entrypointLabels));
 
+    pcToOriginMap.appendItem(m_jit->labelIgnoringWatchpoints(), Origin());
     if (disassembler)
         disassembler->startLatePath(*m_jit);
 
-    // FIXME: Make late paths have Origins: https://bugs.webkit.org/show_bug.cgi?id=153689
-    for (auto& latePath : context.latePaths)
+    for (auto& [origin, latePath] : context.latePaths) {
+        if (m_code.shouldPreserveB3Origins())
+            pcToOriginMap.appendItem(m_jit->labelIgnoringWatchpoints(), origin);
         latePath->run(*m_jit, context);
+    }
 
     if (disassembler)
         disassembler->endLatePath(*m_jit);
+    pcToOriginMap.appendItem(m_jit->labelIgnoringWatchpoints(), Origin());
 }
 
 } } } // namespace JSC::B3::Air
 
+WTF_ALLOW_UNSAFE_BUFFER_USAGE_END
+
 #endif // ENABLE(B3_JIT)
