@@ -33,13 +33,19 @@
 #include "config.h"
 #include "Performance.h"
 
+#include "ContextDestructionObserverInlines.h"
 #include "Document.h"
 #include "DocumentLoader.h"
 #include "Event.h"
+#include "EventCounts.h"
 #include "EventLoop.h"
 #include "EventNames.h"
+#include "ExceptionOr.h"
+#include "LargestContentfulPaint.h"
 #include "LocalFrame.h"
+#include "Logging.h"
 #include "PerformanceEntry.h"
+#include "PerformanceEventTiming.h"
 #include "PerformanceMarkOptions.h"
 #include "PerformanceMeasureOptions.h"
 #include "PerformanceNavigation.h"
@@ -51,19 +57,38 @@
 #include "PerformanceUserTiming.h"
 #include "ResourceResponse.h"
 #include "ScriptExecutionContext.h"
-#include <wtf/IsoMallocInlines.h>
+#include "dom/DOMHighResTimeStamp.h"
+#include <JavaScriptCore/ProfilerSupport.h>
+#include <ranges>
+#include <wtf/SystemTracing.h>
+#include <wtf/TZoneMallocInlines.h>
+#include <wtf/text/StringToIntegerConversion.h>
 
 namespace WebCore {
 
-WTF_MAKE_ISO_ALLOCATED_IMPL(Performance);
+WTF_MAKE_TZONE_OR_ISO_ALLOCATED_IMPL(Performance);
 
-constexpr Seconds highTimePrecision { 20_us };
 static Seconds timePrecision { 1_ms };
 
+static bool isSignpostEnabled()
+{
+    static bool flag = false;
+    static std::once_flag onceKey;
+    std::call_once(onceKey, [&] {
+        const char* value = getenv("WebKitPerformanceSignpostEnabled");
+        if (value) {
+            if (auto result = parseInteger<int>(StringView::fromLatin1(value)); result && result.value())
+                flag = true;
+        }
+    });
+    return flag;
+}
+
 Performance::Performance(ScriptExecutionContext* context, MonotonicTime timeOrigin)
     : ContextDestructionObserver(context)
     , m_resourceTimingBufferFullTimer(*this, &Performance::resourceTimingBufferFullTimerFired) // FIXME: Migrate this to the event loop as well. https://bugs.webkit.org/show_bug.cgi?id=229044
     , m_timeOrigin(timeOrigin)
+    , m_continuousTimeOrigin(timeOrigin.approximateContinuousTime())
 {
     ASSERT(m_timeOrigin);
 }
@@ -94,20 +119,58 @@ ReducedResolutionSeconds Performance::nowInReducedResolutionSeconds() const
 
 Seconds Performance::reduceTimeResolution(Seconds seconds)
 {
-    double resolution = timePrecision.seconds();
-    double reduced = std::floor(seconds.seconds() / resolution) * resolution;
-    return Seconds(reduced);
+    return seconds.reduceTimeResolution(timePrecision);
 }
 
 void Performance::allowHighPrecisionTime()
 {
-    timePrecision = highTimePrecision;
+    timePrecision = Seconds::highTimePrecision();
+}
+
+Seconds Performance::timeResolution()
+{
+    return timePrecision;
+}
+
+Seconds Performance::relativeTimeFromTimeOriginInReducedResolutionSeconds(MonotonicTime timestamp) const
+{
+    return reduceTimeResolution(timestamp - m_timeOrigin);
 }
 
 DOMHighResTimeStamp Performance::relativeTimeFromTimeOriginInReducedResolution(MonotonicTime timestamp) const
 {
-    Seconds seconds = timestamp - m_timeOrigin;
-    return reduceTimeResolution(seconds).milliseconds();
+    return relativeTimeFromTimeOriginInReducedResolutionSeconds(timestamp).milliseconds();
+}
+
+MonotonicTime Performance::monotonicTimeFromRelativeTime(DOMHighResTimeStamp relativeTime) const
+{
+    return m_timeOrigin + Seconds::fromMilliseconds(relativeTime);
+}
+
+ScriptExecutionContext* Performance::scriptExecutionContext() const
+{
+    return ContextDestructionObserver::scriptExecutionContext();
+}
+
+EventCounts* Performance::eventCounts()
+{
+    if (!is<Document>(scriptExecutionContext()))
+        return nullptr;
+
+    ASSERT(isMainThread());
+    // FIXME: stop lazy-initializing m_eventCounts after event
+    // timing stops being gated by a flag:
+    if (!m_eventCounts)
+        lazyInitialize(m_eventCounts, makeUniqueWithoutRefCountedCheck<EventCounts>(this));
+
+    return m_eventCounts.get();
+}
+
+uint64_t Performance::interactionCount()
+{
+    ASSERT(is<Document>(scriptExecutionContext()));
+    ASSERT(isMainThread());
+    return downcast<Document>(*scriptExecutionContext()).window()->interactionCount();
 }
 
 PerformanceNavigation* Performance::navigation()
@@ -117,7 +180,7 @@ PerformanceNavigation* Performance::navigation()
 
     ASSERT(isMainThread());
     if (!m_navigation)
-        m_navigation = PerformanceNavigation::create(downcast<Document>(*scriptExecutionContext()).domWindow());
+        m_navigation = PerformanceNavigation::create(downcast<Document>(*scriptExecutionContext()).window());
     return m_navigation.get();
 }
 
@@ -128,16 +191,16 @@ PerformanceTiming* Performance::timing()
 
     ASSERT(isMainThread());
     if (!m_timing)
-        m_timing = PerformanceTiming::create(downcast<Document>(*scriptExecutionContext()).domWindow());
+        m_timing = PerformanceTiming::create(downcast<Document>(*scriptExecutionContext()).window());
     return m_timing.get();
 }
 
-Vector<RefPtr<PerformanceEntry>> Performance::getEntries() const
+Vector<Ref<PerformanceEntry>> Performance::getEntries() const
 {
-    Vector<RefPtr<PerformanceEntry>> entries;
+    Vector<Ref<PerformanceEntry>> entries;
 
     if (m_navigationTiming)
-        entries.append(m_navigationTiming);
+        entries.append(*m_navigationTiming);
 
     entries.appendVector(m_resourceTimingBuffer);
 
@@ -147,24 +210,31 @@ Vector<RefPtr<PerformanceEntry>> Performance::getEntries() const
     }
 
     if (m_firstContentfulPaint)
-        entries.append(m_firstContentfulPaint);
+        entries.append(*m_firstContentfulPaint);
 
-    std::sort(entries.begin(), entries.end(), PerformanceEntry::startTimeCompareLessThan);
+    // getEntries should not include largest-contentful-paint.
+
+    if (m_firstInput)
+        entries.append(*m_firstInput);
+
+    std::ranges::sort(entries, PerformanceEntry::startTimeCompareLessThan);
     return entries;
 }
 
-Vector<RefPtr<PerformanceEntry>> Performance::getEntriesByType(const String& entryType) const
+Vector<Ref<PerformanceEntry>> Performance::getEntriesByType(const String& entryType) const
 {
-    Vector<RefPtr<PerformanceEntry>> entries;
+    Vector<Ref<PerformanceEntry>> entries;
 
     if (m_navigationTiming && entryType == "navigation"_s)
-        entries.append(m_navigationTiming);
+        entries.append(*m_navigationTiming);
     
     if (entryType == "resource"_s)
         entries.appendVector(m_resourceTimingBuffer);
 
     if (m_firstContentfulPaint && entryType == "paint"_s)
-        entries.append(m_firstContentfulPaint);
+        entries.append(*m_firstContentfulPaint);
+
+    // getEntriesByType should not include largest-contentful-paint.
 
     if (m_userTiming) {
         if (entryType == "mark"_s)
@@ -173,16 +243,19 @@ Vector<RefPtr<PerformanceEntry>> Performance::getEntriesByType(const String& ent
             entries.appendVector(m_userTiming->getMeasures());
     }
 
-    std::sort(entries.begin(), entries.end(), PerformanceEntry::startTimeCompareLessThan);
+    if (entryType == "first-input"_s && m_firstInput)
+        entries.append(*m_firstInput);
+
+    std::ranges::sort(entries, PerformanceEntry::startTimeCompareLessThan);
     return entries;
 }
 
-Vector<RefPtr<PerformanceEntry>> Performance::getEntriesByName(const String& name, const String& entryType) const
+Vector<Ref<PerformanceEntry>> Performance::getEntriesByName(const String& name, const String& entryType) const
 {
-    Vector<RefPtr<PerformanceEntry>> entries;
+    Vector<Ref<PerformanceEntry>> entries;
 
     if (m_navigationTiming && (entryType.isNull() || entryType == "navigation"_s) && name == m_navigationTiming->name())
-        entries.append(m_navigationTiming);
+        entries.append(*m_navigationTiming);
 
     if (entryType.isNull() || entryType == "resource"_s) {
         for (auto& resource : m_resourceTimingBuffer) {
@@ -192,7 +265,9 @@ Vector<RefPtr<PerformanceEntry>> Performance::getEntriesByName(const String& nam
     }
 
     if (m_firstContentfulPaint && (entryType.isNull() || entryType == "paint"_s) && name == "first-contentful-paint"_s)
-        entries.append(m_firstContentfulPaint);
+        entries.append(*m_firstContentfulPaint);
+
+    // getEntriesByName should not include largest-contentful-paint.
 
     if (m_userTiming) {
         if (entryType.isNull() || entryType == "mark"_s)
@@ -201,16 +276,19 @@ Vector<RefPtr<PerformanceEntry>> Performance::getEntriesByName(const String& nam
             entries.appendVector(m_userTiming->getMeasures(name));
     }
 
-    std::sort(entries.begin(), entries.end(), PerformanceEntry::startTimeCompareLessThan);
+    if (entryType.isNull() || entryType == "first-input"_s) {
+        if (m_firstInput && name == m_firstInput->name())
+            entries.append(*m_firstInput);
+    }
+
+    std::ranges::sort(entries, PerformanceEntry::startTimeCompareLessThan);
     return entries;
 }
 
-void Performance::appendBufferedEntriesByType(const String& entryType, Vector<RefPtr<PerformanceEntry>>& entries, PerformanceObserver& observer) const
+void Performance::appendBufferedEntriesByType(const String& entryType, Vector<Ref<PerformanceEntry>>& entries, PerformanceObserver& observer) const
 {
-    if (m_navigationTiming
-        && entryType == "navigation"_s
-        && !observer.hasNavigationTiming()) {
-        entries.append(m_navigationTiming);
+    if (m_navigationTiming && entryType == "navigation"_s && !observer.hasNavigationTiming()) {
+        entries.append(*m_navigationTiming);
         observer.addedNavigationTiming();
     }
 
@@ -218,7 +296,16 @@ void Performance::appendBufferedEntriesByType(const String& entryType, Vector<Re
         entries.appendVector(m_resourceTimingBuffer);
 
     if (entryType == "paint"_s && m_firstContentfulPaint)
-        entries.append(m_firstContentfulPaint);
+        entries.append(*m_firstContentfulPaint);
+
+    if (entryType == "largest-contentful-paint"_s && m_largestContentfulPaint)
+        entries.append(*m_largestContentfulPaint);
+
+    if (entryType == "event"_s)
+        entries.appendVector(m_eventTimingBuffer);
+
+    if (entryType == "first-input"_s && m_firstInput)
+        entries.append(*m_firstInput);
 
     if (m_userTiming) {
         if (entryType.isNull() || entryType == "mark"_s)
@@ -228,6 +315,52 @@ void Performance::appendBufferedEntriesByType(const String& entryType, Vector<Re
     }
 }
 
+void Performance::countEvent(EventType type)
+{
+    ASSERT(isMainThread());
+    eventCounts()->add(type);
+}
+
+void Performance::processEventEntry(const PerformanceEventTimingCandidate& candidate)
+{
+    // Constants to avoid the need to round to the appropriate resolution
+    // (PerformanceEventTiming::durationResolution) when filtering candidates:
+    static constexpr Seconds minDurationCutoffBeforeRounding = PerformanceEventTiming::minimumDurationThreshold - (PerformanceEventTiming::durationResolution / 2);
+    static constexpr Seconds defaultDurationCutoffBeforeRounding = PerformanceEventTiming::defaultDurationThreshold - (PerformanceEventTiming::durationResolution / 2);
+
+    // The event timing spec requires us to set first-input and call
+    // setDispatchedInputEvent() when an entry with nonzero interactionID has
+    // its duration set, and only if hasDispatchedInputEvent() is false. This, however,
+    // causes inconsistent behavior: a pointerdown event that had its duration assigned
+    // before receiving an interactionID wouldn't qualify for first-input.
+    //
+    // We instead set first-input and call setDispatchedInputEvent() here; ongoing
+    // spec discussion at https://github.com/w3c/event-timing/issues/159 :
+    if (!m_firstInput && !candidate.interactionID.isUnassigned()) {
+        m_firstInput = PerformanceEventTiming::create(candidate, true);
+        queueEntry(*m_firstInput);
+        if (RefPtr document = dynamicDowncast<Document>(*scriptExecutionContext())) {
+            if (RefPtr window = document->window())
+                window->setDispatchedInputEvent();
+        }
+    }
+
+    if (candidate.duration < minDurationCutoffBeforeRounding)
+        return;
+
+    // FIXME: early return more often by keeping track of m_observers; we
+    // should keep track of the minimum defaultThreshold and whether any
+    // observers are interested in 'event' entries:
+    if (candidate.duration <= defaultDurationCutoffBeforeRounding && !m_observers.size())
+        return;
+
+    auto entry = PerformanceEventTiming::create(candidate);
+    if (m_eventTimingBuffer.size() < m_eventTimingBufferSize && candidate.duration > defaultDurationCutoffBeforeRounding)
+        m_eventTimingBuffer.append(entry);
+
+    queueEntry(entry);
+}
+
 void Performance::clearResourceTimings()
 {
     m_resourceTimingBuffer.clear();
@@ -240,25 +373,38 @@ void Performance::setResourceTimingBufferSize(unsigned size)
     m_resourceTimingBufferFullFlag = false;
 }
 
-void Performance::reportFirstContentfulPaint()
+void Performance::reportFirstContentfulPaint(DOMHighResTimeStamp timestamp)
 {
     ASSERT(!m_firstContentfulPaint);
-    m_firstContentfulPaint = PerformancePaintTiming::createFirstContentfulPaint(now());
+    m_firstContentfulPaint = PerformancePaintTiming::createFirstContentfulPaint(timestamp);
     queueEntry(*m_firstContentfulPaint);
 }
 
+void Performance::enqueueLargestContentfulPaint(Ref<LargestContentfulPaint>&& paintEntry)
+{
+    m_largestContentfulPaint = RefPtr { WTFMove(paintEntry) };
+    queueEntry(*m_largestContentfulPaint);
+}
+
 void Performance::addNavigationTiming(DocumentLoader& documentLoader, Document& document, CachedResource& resource, const DocumentLoadTiming& timing, const NetworkLoadMetrics& metrics)
 {
-    ASSERT(document.settings().performanceNavigationTimingAPIEnabled());
     m_navigationTiming = PerformanceNavigationTiming::create(m_timeOrigin, resource, timing, metrics, document.eventTiming(), document.securityOrigin(), documentLoader.triggeringAction().type());
 }
 
-void Performance::navigationFinished(const NetworkLoadMetrics& metrics)
+void Performance::documentLoadFinished(const NetworkLoadMetrics& metrics)
 {
     if (!m_navigationTiming)
         return;
-    m_navigationTiming->navigationFinished(metrics);
 
+    m_navigationTiming->documentLoadFinished(metrics);
+}
+
+void Performance::navigationFinished(MonotonicTime loadEventEnd)
+{
+    if (!m_navigationTiming)
+        return;
+
+    m_navigationTiming->documentLoadTiming().setLoadEventEnd(loadEventEnd);
     queueEntry(*m_navigationTiming);
 }
 
@@ -304,7 +450,7 @@ void Performance::resourceTimingBufferFullTimerFired()
     while (!m_backupResourceTimingBuffer.isEmpty()) {
         auto beforeCount = m_backupResourceTimingBuffer.size();
 
-        auto backupBuffer = WTFMove(m_backupResourceTimingBuffer);
+        auto backupBuffer = std::exchange(m_backupResourceTimingBuffer, { });
         ASSERT(m_backupResourceTimingBuffer.isEmpty());
 
         if (isResourceTimingBufferFull()) {
@@ -314,22 +460,20 @@ void Performance::resourceTimingBufferFullTimerFired()
 
         if (m_resourceTimingBufferFullFlag) {
             for (auto& entry : backupBuffer)
-                queueEntry(*entry);
+                queueEntry(entry);
             // Dispatching resourcetimingbufferfull event may have inserted more entries.
-            for (auto& entry : m_backupResourceTimingBuffer)
-                queueEntry(*entry);
-            m_backupResourceTimingBuffer.clear();
+            for (auto& entry : std::exchange(m_backupResourceTimingBuffer, { }))
+                queueEntry(entry);
             break;
         }
 
         // More entries may have added while dispatching resourcetimingbufferfull event.
-        backupBuffer.appendVector(m_backupResourceTimingBuffer);
-        m_backupResourceTimingBuffer.clear();
+        backupBuffer.appendVector(std::exchange(m_backupResourceTimingBuffer, { }));
 
         for (auto& entry : backupBuffer) {
             if (!isResourceTimingBufferFull()) {
                 m_resourceTimingBuffer.append(entry.copyRef());
-                queueEntry(*entry);
+                queueEntry(entry);
             } else
                 m_backupResourceTimingBuffer.append(entry.copyRef());
         }
@@ -373,6 +517,33 @@ ExceptionOr<Ref<PerformanceMeasure>> Performance::measure(JSC::JSGlobalObject& g
     if (measure.hasException())
         return measure.releaseException();
 
+    if (isSignpostEnabled()) {
+        Ref entry { measure.returnValue() };
+        auto message = measureName.utf8();
+#if OS(DARWIN)
+        {
+            auto startTime = m_continuousTimeOrigin + Seconds::fromMilliseconds(entry->startTime());
+            auto endTime = m_continuousTimeOrigin + Seconds::fromMilliseconds(entry->startTime() + entry->duration());
+            uint64_t platformStartTime = startTime.toMachContinuousTime();
+            uint64_t platformEndTime = endTime.toMachContinuousTime();
+            uint64_t correctedStartTime = std::min(platformStartTime, platformEndTime);
+            uint64_t correctedEndTime = std::max(platformStartTime, platformEndTime);
+            // Because signpost intervals are closed invervals [start, end], we decrease the endTime by 1 if startTime and endTime is not the same.
+            if (correctedStartTime != correctedEndTime)
+                correctedEndTime -= 1;
+
+            WTFBeginSignpostAlwaysWithSpecificTime(entry.ptr(), WebKitPerformance, correctedStartTime, "%" PUBLIC_LOG_STRING, message.data());
+            WTFEndSignpostAlwaysWithSpecificTime(entry.ptr(), WebKitPerformance, correctedEndTime, "%" PUBLIC_LOG_STRING, message.data());
+        }
+#endif
+        {
+            auto timeOrigin = m_continuousTimeOrigin.approximateMonotonicTime();
+            auto startTime = timeOrigin + Seconds::fromMilliseconds(entry->startTime());
+            auto endTime = timeOrigin + Seconds::fromMilliseconds(entry->startTime() + entry->duration());
+            JSC::ProfilerSupport::markInterval(entry.ptr(), JSC::ProfilerSupport::Category::WebKitPerformanceSignpost, startTime, endTime, WTFMove(message));
+        }
+    }
+
     queueEntry(measure.returnValue().get());
     return measure.releaseReturnValue();
 }
@@ -394,13 +565,6 @@ void Performance::removeAllObservers()
 void Performance::registerPerformanceObserver(PerformanceObserver& observer)
 {
     m_observers.add(&observer);
-
-    if (m_navigationTiming
-        && observer.typeFilter().contains(PerformanceEntry::Type::Navigation)
-        && !observer.hasNavigationTiming()) {
-        observer.queueEntry(*m_navigationTiming);
-        observer.addedNavigationTiming();
-    }
 }
 
 void Performance::unregisterPerformanceObserver(PerformanceObserver& observer)
@@ -408,17 +572,18 @@ void Performance::unregisterPerformanceObserver(PerformanceObserver& observer)
     m_observers.remove(&observer);
 }
 
-void Performance::scheduleNavigationObservationTaskIfNeeded()
-{
-    if (m_navigationTiming)
-        scheduleTaskIfNeeded();
-}
-
 void Performance::queueEntry(PerformanceEntry& entry)
 {
     bool shouldScheduleTask = false;
     for (auto& observer : m_observers) {
-        if (observer->typeFilter().contains(entry.performanceEntryType())) {
+        bool isObserverInterested = observer->typeFilter().contains(entry.performanceEntryType());
+        if (entry.performanceEntryType() == PerformanceEntry::Type::Event && entry.duration() < observer->durationThreshold().milliseconds())
+            isObserverInterested = false;
+
+        if (entry.performanceEntryType() == PerformanceEntry::Type::Navigation && observer->hasNavigationTiming())
+            isObserverInterested = false;
+
+        if (isObserverInterested) {
             observer->queueEntry(entry);
             shouldScheduleTask = true;
         }
@@ -427,25 +592,26 @@ void Performance::queueEntry(PerformanceEntry& entry)
     if (!shouldScheduleTask)
         return;
 
+    LOG_WITH_STREAM(PerformanceTimeline, stream << "PerformanceEntry of type " << entry.name() << " dispatched to interested observer at t=" << now());
     scheduleTaskIfNeeded();
 }
 
 void Performance::scheduleTaskIfNeeded()
 {
-    if (m_hasScheduledTimingBufferDeliveryTask)
+    if (m_hasScheduledDeliveryTask)
         return;
 
-    auto* context = scriptExecutionContext();
+    RefPtr context = scriptExecutionContext();
     if (!context)
         return;
 
-    m_hasScheduledTimingBufferDeliveryTask = true;
+    m_hasScheduledDeliveryTask = true;
     context->eventLoop().queueTask(TaskSource::PerformanceTimeline, [protectedThis = Ref { *this }, this] {
-        auto* context = scriptExecutionContext();
+        RefPtr context = scriptExecutionContext();
         if (!context)
             return;
 
-        m_hasScheduledTimingBufferDeliveryTask = false;
+        m_hasScheduledDeliveryTask = false;
         for (auto& observer : copyToVector(m_observers))
             observer->deliver();
     });
