@@ -18,11 +18,10 @@
  * Boston, MA 02110-1301, USA.
  */
 
-#ifndef AutoTableLayout_h
-#define AutoTableLayout_h
+#pragma once
 
 #include "LayoutUnit.h"
-#include "Length.h"
+#include "StylePreferredSize.h"
 #include "TableLayout.h"
 #include <wtf/Vector.h>
 
@@ -36,21 +35,23 @@ public:
     explicit AutoTableLayout(RenderTable*);
     virtual ~AutoTableLayout();
 
-    virtual void computeIntrinsicLogicalWidths(LayoutUnit& minWidth, LayoutUnit& maxWidth) override;
-    virtual void applyPreferredLogicalWidthQuirks(LayoutUnit& minWidth, LayoutUnit& maxWidth) const override;
-    virtual void layout() override;
+    void computeIntrinsicLogicalWidths(LayoutUnit& minWidth, LayoutUnit& maxWidth, TableIntrinsics) override;
+    LayoutUnit scaledWidthFromPercentColumns() const override { return m_scaledWidthFromPercentColumns; }
+    void applyPreferredLogicalWidthQuirks(LayoutUnit& minWidth, LayoutUnit& maxWidth) const override;
+    void layout() override;
 
 private:
     void fullRecalc();
     void recalcColumn(unsigned effCol);
 
     float calcEffectiveLogicalWidth();
+    template<typename> float shrinkCellWidthForType(float available);
 
     void insertSpanCell(RenderTableCell*);
 
     struct Layout {
-        Length logicalWidth;
-        Length effectiveLogicalWidth;
+        Style::PreferredSize logicalWidth { CSS::Keyword::Auto { } };
+        Style::PreferredSize effectiveLogicalWidth { CSS::Keyword::Auto { } };
         float minLogicalWidth { 0 };
         float maxLogicalWidth { 0 };
         float effectiveMinLogicalWidth { 0 };
@@ -59,12 +60,11 @@ private:
         bool emptyCellsOnly { true };
     };
 
-    Vector<Layout, 4> m_layoutStruct;
-    Vector<RenderTableCell*, 4> m_spanCells;
+    Vector<Layout> m_layoutStruct;
+    Vector<RenderTableCell*> m_spanCells;
     bool m_hasPercent : 1;
     mutable bool m_effectiveLogicalWidthDirty : 1;
+    LayoutUnit m_scaledWidthFromPercentColumns;
 };
 
 } // namespace WebCore
-
-#endif // AutoTableLayout_h
