@@ -1,6 +1,6 @@
 /*
- * Copyright (C) 2008-2022 Apple Inc. All Rights Reserved.
- * Copyright (C) 2009, 2011 Google Inc. All Rights Reserved.
+ * Copyright (C) 2008-2025 Apple Inc. All rights reserved.
+ * Copyright (C) 2009, 2011 Google Inc. All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -34,22 +34,25 @@
 #include "CacheStorageProvider.h"
 #include "CommonVM.h"
 #include "ContentSecurityPolicy.h"
+#include "CrossOriginMode.h"
 #include "Crypto.h"
-#include "DocumentInlines.h"
+#include "CryptoKeyData.h"
+#include "DOMTimer.h"
 #include "FontCustomPlatformData.h"
 #include "FontFaceSet.h"
-#include "GCController.h"
+#include "FrameConsoleClient.h"
+#include "GarbageCollectionController.h"
 #include "IDBConnectionProxy.h"
 #include "ImageBitmapOptions.h"
 #include "InspectorInstrumentation.h"
 #include "JSDOMExceptionHandling.h"
 #include "Logging.h"
 #include "NotImplemented.h"
-#include "PageConsoleClient.h"
 #include "Performance.h"
 #include "RTCDataChannelRemoteHandlerConnection.h"
 #include "ReportingScope.h"
 #include "ScheduledAction.h"
+#include "ScriptExecutionContextInlines.h"
 #include "ScriptSourceCode.h"
 #include "SecurityOrigin.h"
 #include "SecurityOriginPolicy.h"
@@ -77,11 +80,15 @@
 #include "WorkerThread.h"
 #include <JavaScriptCore/ScriptArguments.h>
 #include <JavaScriptCore/ScriptCallStack.h>
-#include <wtf/IsoMallocInlines.h>
 #include <wtf/Lock.h>
+#include <wtf/TZoneMallocInlines.h>
 #include <wtf/WorkQueue.h>
 #include <wtf/threads/BinarySemaphore.h>
 
+#if ENABLE(WEBDRIVER_BIDI)
+#include "AutomationInstrumentation.h"
+#endif
+
 namespace WebCore {
 using namespace Inspector;
 
@@ -93,16 +100,16 @@ static HashSet<ScriptExecutionContextIdentifier>& allWorkerGlobalScopeIdentifier
     return identifiers;
 }
 
-static WorkQueue& sharedFileSystemStorageQueue()
+static WorkQueue& sharedFileSystemStorageQueueSingleton()
 {
     static NeverDestroyed<Ref<WorkQueue>> queue(WorkQueue::create("Shared File System Storage Queue"_s,  WorkQueue::QOS::Default));
     return queue.get();
 }
 
-WTF_MAKE_ISO_ALLOCATED_IMPL(WorkerGlobalScope);
+WTF_MAKE_TZONE_OR_ISO_ALLOCATED_IMPL(WorkerGlobalScope);
 
 WorkerGlobalScope::WorkerGlobalScope(WorkerThreadType type, const WorkerParameters& params, Ref<SecurityOrigin>&& origin, WorkerThread& thread, Ref<SecurityOrigin>&& topOrigin, IDBClient::IDBConnectionProxy* connectionProxy, SocketProvider* socketProvider, std::unique_ptr<WorkerClient>&& workerClient)
-    : WorkerOrWorkletGlobalScope(type, params.sessionID, isMainThread() ? Ref { commonVM() } : JSC::VM::create(), params.referrerPolicy, &thread, params.noiseInjectionHashSalt, params.advancedPrivacyProtections, params.clientIdentifier)
+    : WorkerOrWorkletGlobalScope(type, params.sessionID, isMainThread() ? Ref { commonVM() } : JSC::VM::create(JSC::HeapType::Medium), params.referrerPolicy, &thread, params.noiseInjectionHashSalt, params.advancedPrivacyProtections, params.clientIdentifier)
     , m_url(params.scriptURL)
     , m_ownerURL(params.ownerURL)
     , m_inspectorIdentifier(params.inspectorIdentifier)
@@ -137,7 +144,7 @@ WorkerGlobalScope::WorkerGlobalScope(WorkerThreadType type, const WorkerParamete
 
 WorkerGlobalScope::~WorkerGlobalScope()
 {
-    ASSERT(thread().thread() == &Thread::current());
+    ASSERT(thread()->thread() == &Thread::currentSingleton());
 
     {
         Locker locker { allWorkerGlobalScopeIdentifiersLock };
@@ -148,13 +155,13 @@ WorkerGlobalScope::~WorkerGlobalScope()
     m_crypto = nullptr;
 
     // Notify proxy that we are going away. This can free the WorkerThread object, so do not access it after this.
-    if (auto* workerReportingProxy = thread().workerReportingProxy())
+    if (auto* workerReportingProxy = thread()->workerReportingProxy())
         workerReportingProxy->workerGlobalScopeDestroyed();
 }
 
 String WorkerGlobalScope::origin() const
 {
-    auto* securityOrigin = this->securityOrigin();
+    RefPtr securityOrigin = this->securityOrigin();
     return securityOrigin ? securityOrigin->toString() : emptyString();
 }
 
@@ -162,25 +169,28 @@ void WorkerGlobalScope::prepareForDestruction()
 {
     WorkerOrWorkletGlobalScope::prepareForDestruction();
 
-    removeSupplement(WindowOrWorkerGlobalScopeTrustedTypes::workerGlobalSupplementName());
+    if (auto* trustedTypes = downcast<WorkerGlobalScopeTrustedTypes>(requireSupplement(WorkerGlobalScopeTrustedTypes::supplementName())))
+        trustedTypes->prepareForDestruction();
 
     if (settingsValues().serviceWorkersEnabled)
         swClientConnection().unregisterServiceWorkerClient(identifier());
 
-    stopIndexedDatabase();
+    if (m_connectionProxy)
+        m_connectionProxy->abortActivitiesForCurrentThread();
 
     if (m_storageConnection)
         m_storageConnection->scopeClosed();
 
-    if (m_fileSystemStorageConnection)
-        m_fileSystemStorageConnection->scopeClosed();
+    if (RefPtr fileSystemStorageConnection = m_fileSystemStorageConnection)
+        fileSystemStorageConnection->scopeClosed();
 }
 
 void WorkerGlobalScope::removeAllEventListeners()
 {
     WorkerOrWorkletGlobalScope::removeAllEventListeners();
-    m_performance->removeAllEventListeners();
-    m_performance->removeAllObservers();
+    Ref performance = *m_performance;
+    performance->removeAllEventListeners();
+    performance->removeAllObservers();
     m_reportingScope->removeAllObservers();
 }
 
@@ -217,11 +227,16 @@ SocketProvider* WorkerGlobalScope::socketProvider()
     return m_socketProvider.get();
 }
 
+RefPtr<SocketProvider> WorkerGlobalScope::protectedSocketProvider()
+{
+    return socketProvider();
+}
+
 RefPtr<RTCDataChannelRemoteHandlerConnection> WorkerGlobalScope::createRTCDataChannelRemoteHandlerConnection()
 {
     RefPtr<RTCDataChannelRemoteHandlerConnection> connection;
     callOnMainThreadAndWait([workerThread = Ref { thread() }, &connection]() mutable {
-        if (auto* workerLoaderProxy = workerThread->workerLoaderProxy())
+        if (CheckedPtr workerLoaderProxy = workerThread->workerLoaderProxy())
             connection = workerLoaderProxy->createRTCDataChannelRemoteHandlerConnection();
     });
     ASSERT(connection);
@@ -239,12 +254,6 @@ GraphicsClient* WorkerGlobalScope::graphicsClient()
     return workerClient();
 }
 
-void WorkerGlobalScope::stopIndexedDatabase()
-{
-    if (m_connectionProxy)
-        m_connectionProxy->forgetActivityForCurrentThread();
-}
-
 void WorkerGlobalScope::suspend()
 {
     if (m_connectionProxy)
@@ -266,14 +275,14 @@ void WorkerGlobalScope::resume()
 WorkerStorageConnection& WorkerGlobalScope::storageConnection()
 {
     if (!m_storageConnection)
-        m_storageConnection = WorkerStorageConnection::create(*this);
+        lazyInitialize(m_storageConnection, WorkerStorageConnection::create(*this));
 
     return *m_storageConnection;
 }
 
 void WorkerGlobalScope::postFileSystemStorageTask(Function<void()>&& task)
 {
-    sharedFileSystemStorageQueue().dispatch(WTFMove(task));
+    sharedFileSystemStorageQueueSingleton().dispatch(WTFMove(task));
 }
 
 WorkerFileSystemStorageConnection& WorkerGlobalScope::getFileSystemStorageConnection(Ref<FileSystemStorageConnection>&& mainThreadConnection)
@@ -281,7 +290,7 @@ WorkerFileSystemStorageConnection& WorkerGlobalScope::getFileSystemStorageConnec
     if (!m_fileSystemStorageConnection)
         m_fileSystemStorageConnection = WorkerFileSystemStorageConnection::create(*this, WTFMove(mainThreadConnection));
     else if (m_fileSystemStorageConnection->mainThreadConnection() != mainThreadConnection.ptr()) {
-        m_fileSystemStorageConnection->connectionClosed();
+        Ref { *m_fileSystemStorageConnection }->connectionClosed();
         m_fileSystemStorageConnection = WorkerFileSystemStorageConnection::create(*this, WTFMove(mainThreadConnection));
     }
 
@@ -296,7 +305,7 @@ WorkerFileSystemStorageConnection* WorkerGlobalScope::fileSystemStorageConnectio
 WorkerLocation& WorkerGlobalScope::location() const
 {
     if (!m_location)
-        m_location = WorkerLocation::create(URL { m_url }, origin());
+        lazyInitialize(m_location, WorkerLocation::create(URL { m_url }, origin()));
     return *m_location;
 }
 
@@ -313,7 +322,7 @@ void WorkerGlobalScope::close()
         ASSERT_WITH_SECURITY_IMPLICATION(is<WorkerGlobalScope>(context));
         WorkerGlobalScope& workerGlobalScope = downcast<WorkerGlobalScope>(context);
         // Notify parent that this context is closed. Parent is responsible for calling WorkerThread::stop().
-        if (auto* workerReportingProxy = workerGlobalScope.thread().workerReportingProxy())
+        if (auto* workerReportingProxy = workerGlobalScope.thread()->workerReportingProxy())
             workerReportingProxy->workerGlobalScopeClosed();
     } });
 }
@@ -321,10 +330,15 @@ void WorkerGlobalScope::close()
 WorkerNavigator& WorkerGlobalScope::navigator()
 {
     if (!m_navigator)
-        m_navigator = WorkerNavigator::create(*this, m_userAgent, m_isOnline);
+        lazyInitialize(m_navigator, WorkerNavigator::create(*this, m_userAgent, m_isOnline));
     return *m_navigator;
 }
 
+Ref<WorkerNavigator> WorkerGlobalScope::protectedNavigator()
+{
+    return navigator();
+}
+
 void WorkerGlobalScope::setIsOnline(bool isOnline)
 {
     m_isOnline = isOnline;
@@ -368,7 +382,7 @@ void WorkerGlobalScope::clearInterval(int timeoutId)
     DOMTimer::removeById(*this, timeoutId);
 }
 
-ExceptionOr<void> WorkerGlobalScope::importScripts(const FixedVector<std::variant<RefPtr<TrustedScriptURL>, String>>& urls)
+ExceptionOr<void> WorkerGlobalScope::importScripts(const FixedVector<Variant<RefPtr<TrustedScriptURL>, String>>& urls)
 {
     ASSERT(contentSecurityPolicy());
 
@@ -432,7 +446,7 @@ ExceptionOr<void> WorkerGlobalScope::importScripts(const FixedVector<std::varian
         {
             NakedPtr<JSC::Exception> exception;
             ScriptSourceCode sourceCode(scriptLoader->script(), URL(scriptLoader->responseURL()), scriptLoader->isRedirected() ? URL(scriptLoader->url()) : URL());
-            sourceProvider = static_cast<ScriptBufferSourceProvider&>(sourceCode.provider());
+            sourceProvider = downcast<ScriptBufferSourceProvider>(sourceCode.provider());
             script()->evaluate(sourceCode, exception);
             if (exception) {
                 if (mutedErrors)
@@ -455,7 +469,7 @@ EventTarget* WorkerGlobalScope::errorEventTarget()
 
 void WorkerGlobalScope::logExceptionToConsole(const String& errorMessage, const String& sourceURL, int lineNumber, int columnNumber, RefPtr<ScriptCallStack>&&)
 {
-    if (auto* workerReportingProxy = thread().workerReportingProxy())
+    if (auto* workerReportingProxy = thread()->workerReportingProxy())
         workerReportingProxy->postExceptionToWorkerObject(errorMessage, lineNumber, columnNumber, sourceURL);
 }
 
@@ -467,9 +481,12 @@ void WorkerGlobalScope::addConsoleMessage(std::unique_ptr<Inspector::ConsoleMess
     }
 
     auto sessionID = this->sessionID();
-    if (UNLIKELY(settingsValues().logsPageMessagesToSystemConsoleEnabled && sessionID && !sessionID->isEphemeral()))
-        PageConsoleClient::logMessageToSystemConsole(*message);
+    if (settingsValues().logsPageMessagesToSystemConsoleEnabled && sessionID && !sessionID->isEphemeral()) [[unlikely]]
+        FrameConsoleClient::logMessageToSystemConsole(*message);
 
+#if ENABLE(WEBDRIVER_BIDI)
+    AutomationInstrumentation::addMessageToConsole(message);
+#endif
     InspectorInstrumentation::addMessageToConsole(*this, WTFMove(message));
 }
 
@@ -490,20 +507,24 @@ void WorkerGlobalScope::addMessage(MessageSource source, MessageLevel level, con
         message = makeUnique<Inspector::ConsoleMessage>(source, MessageType::Log, level, messageText, callStack.releaseNonNull(), requestIdentifier);
     else
         message = makeUnique<Inspector::ConsoleMessage>(source, MessageType::Log, level, messageText, sourceURL, lineNumber, columnNumber, state, requestIdentifier);
+
+#if ENABLE(WEBDRIVER_BIDI)
+    AutomationInstrumentation::addMessageToConsole(message);
+#endif
     InspectorInstrumentation::addMessageToConsole(*this, WTFMove(message));
 }
 
-std::optional<Vector<uint8_t>> WorkerGlobalScope::wrapCryptoKey(const Vector<uint8_t>& key)
+std::optional<Vector<uint8_t>> WorkerGlobalScope::serializeAndWrapCryptoKey(CryptoKeyData&& keyData)
 {
     Ref protectedThis { *this };
-    auto* workerLoaderProxy = thread().workerLoaderProxy();
+    auto* workerLoaderProxy = thread()->workerLoaderProxy();
     if (!workerLoaderProxy)
         return std::nullopt;
 
     BinarySemaphore semaphore;
     std::optional<Vector<uint8_t>> wrappedKey;
-    workerLoaderProxy->postTaskToLoader([&semaphore, &key, &wrappedKey](auto& context) {
-        wrappedKey = context.wrapCryptoKey(key);
+    workerLoaderProxy->postTaskToLoader([&semaphore, &wrappedKey, keyData = crossThreadCopy(WTFMove(keyData))](auto& context) mutable  {
+        wrappedKey = context.serializeAndWrapCryptoKey(WTFMove(keyData));
         semaphore.signal();
     });
     semaphore.wait();
@@ -513,7 +534,7 @@ std::optional<Vector<uint8_t>> WorkerGlobalScope::wrapCryptoKey(const Vector<uin
 std::optional<Vector<uint8_t>> WorkerGlobalScope::unwrapCryptoKey(const Vector<uint8_t>& wrappedKey)
 {
     Ref protectedThis { *this };
-    auto* workerLoaderProxy = thread().workerLoaderProxy();
+    auto* workerLoaderProxy = thread()->workerLoaderProxy();
     if (!workerLoaderProxy)
         return std::nullopt;
 
@@ -551,14 +572,14 @@ CacheStorageConnection& WorkerGlobalScope::cacheStorageConnection()
         callOnMainThreadAndWait([workerThread = Ref { thread() }, &mainThreadConnection]() mutable {
             if (workerThread->runLoop().terminated())
                 return;
-            if (auto* workerLoaderProxy = workerThread->workerLoaderProxy())
+            if (CheckedPtr workerLoaderProxy = workerThread->workerLoaderProxy())
                 mainThreadConnection = workerLoaderProxy->createCacheStorageConnection();
         });
         if (!mainThreadConnection) {
             RELEASE_LOG_INFO(ServiceWorker, "Creating worker dummy CacheStorageConnection");
             mainThreadConnection = CacheStorageProvider::DummyCacheStorageConnection::create();
         }
-        m_cacheStorageConnection = mainThreadConnection.releaseNonNull();
+        lazyInitialize(m_cacheStorageConnection, mainThreadConnection.releaseNonNull());
     }
     return *m_cacheStorageConnection;
 }
@@ -566,14 +587,14 @@ CacheStorageConnection& WorkerGlobalScope::cacheStorageConnection()
 MessagePortChannelProvider& WorkerGlobalScope::messagePortChannelProvider()
 {
     if (!m_messagePortChannelProvider)
-        m_messagePortChannelProvider = makeUnique<WorkerMessagePortChannelProvider>(*this);
+        lazyInitialize(m_messagePortChannelProvider, WorkerMessagePortChannelProvider::create(*this));
     return *m_messagePortChannelProvider;
 }
 
 WorkerSWClientConnection& WorkerGlobalScope::swClientConnection()
 {
     if (!m_swClientConnection)
-        m_swClientConnection = WorkerSWClientConnection::create(*this);
+        lazyInitialize(m_swClientConnection, WorkerSWClientConnection::create(*this));
     return *m_swClientConnection;
 }
 
@@ -590,14 +611,14 @@ void WorkerGlobalScope::createImageBitmap(ImageBitmap::Source&& source, int sx,
 CSSValuePool& WorkerGlobalScope::cssValuePool()
 {
     if (!m_cssValuePool)
-        m_cssValuePool = makeUnique<CSSValuePool>();
+        lazyInitialize(m_cssValuePool, makeUnique<CSSValuePool>());
     return *m_cssValuePool;
 }
 
 CSSFontSelector* WorkerGlobalScope::cssFontSelector()
 {
     if (!m_cssFontSelector)
-        m_cssFontSelector = CSSFontSelector::create(*this);
+        lazyInitialize(m_cssFontSelector, CSSFontSelector::create(*this));
     return m_cssFontSelector.get();
 }
 
@@ -617,9 +638,9 @@ void WorkerGlobalScope::beginLoadingFontSoon(FontLoadRequest& request)
     downcast<WorkerFontLoadRequest>(request).load(*this);
 }
 
-WorkerThread& WorkerGlobalScope::thread() const
+Ref<WorkerThread> WorkerGlobalScope::thread() const
 {
-    return *static_cast<WorkerThread*>(workerOrWorkletThread());
+    return downcast<WorkerThread>(workerOrWorkletThread()).releaseNonNull();
 }
 
 void WorkerGlobalScope::releaseMemory(Synchronous synchronous)
@@ -649,11 +670,7 @@ void WorkerGlobalScope::deleteJSCodeAndGC(Synchronous synchronous)
         return;
     }
 #endif
-#if USE(CF) || USE(GLIB)
     vm().heap.reportAbandonedObjectGraph();
-#else
-    vm().heap.collectNow(JSC::Async, JSC::CollectionScope::Full);
-#endif
 }
 
 void WorkerGlobalScope::releaseMemoryInWorkers(Synchronous synchronous)
@@ -671,7 +688,7 @@ void WorkerGlobalScope::dumpGCHeapForWorkers()
     Locker locker { allWorkerGlobalScopeIdentifiersLock };
     for (auto& globalScopeIdentifier : allWorkerGlobalScopeIdentifiers()) {
         postTaskTo(globalScopeIdentifier, [](auto& context) {
-            GCController::dumpHeapForVM(downcast<WorkerGlobalScope>(context).vm());
+            GarbageCollectionController::dumpHeapForVM(downcast<WorkerGlobalScope>(context).vm());
         });
     }
 }
@@ -691,7 +708,7 @@ void WorkerGlobalScope::addImportedScriptSourceProvider(const URL& url, ScriptBu
 
 void WorkerGlobalScope::reportErrorToWorkerObject(const String& errorMessage)
 {
-    if (auto* workerReportingProxy = thread().workerReportingProxy())
+    if (auto* workerReportingProxy = thread()->workerReportingProxy())
         workerReportingProxy->reportErrorToWorkerObject(errorMessage);
 }
 
@@ -699,12 +716,12 @@ void WorkerGlobalScope::clearDecodedScriptData()
 {
     ASSERT(isContextThread());
 
-    if (m_mainScriptSourceProvider)
-        m_mainScriptSourceProvider->clearDecodedData();
+    if (RefPtr provider = m_mainScriptSourceProvider.get())
+        provider->clearDecodedData();
 
     for (auto& sourceProviders : m_importedScriptsSourceProviders.values()) {
-        for (auto& sourceProvider : sourceProviders)
-            sourceProvider.clearDecodedData();
+        for (Ref sourceProvider : sourceProviders)
+            sourceProvider->clearDecodedData();
     }
 }
 
@@ -717,15 +734,17 @@ void WorkerGlobalScope::updateSourceProviderBuffers(const ScriptBuffer& mainScri
 {
     ASSERT(isContextThread());
 
-    if (mainScript && m_mainScriptSourceProvider)
-        m_mainScriptSourceProvider->tryReplaceScriptBuffer(mainScript);
+    if (mainScript) {
+        if (RefPtr provider = m_mainScriptSourceProvider.get())
+            provider->tryReplaceScriptBuffer(mainScript);
+    }
 
     for (auto& pair : importedScripts) {
         auto it = m_importedScriptsSourceProviders.find(pair.key);
         if (it == m_importedScriptsSourceProviders.end())
             continue;
-        for (auto& sourceProvider : it->value)
-            sourceProvider.tryReplaceScriptBuffer(pair.value);
+        for (Ref sourceProvider : it->value)
+            sourceProvider->tryReplaceScriptBuffer(pair.value);
     }
 }
 
@@ -749,7 +768,7 @@ String WorkerGlobalScope::endpointURIForToken(const String& token) const
     return reportingScope().endpointURIForToken(token);
 }
 
-void WorkerGlobalScope::sendReportToEndpoints(const URL&, const Vector<String>& /*endpointURIs*/, const Vector<String>& /*endpointTokens*/, Ref<FormData>&&, ViolationReportType)
+void WorkerGlobalScope::sendReportToEndpoints(const URL&, std::span<const String> /*endpointURIs*/, std::span<const String> /*endpointTokens*/, Ref<FormData>&&, ViolationReportType)
 {
     notImplemented();
 }
