@@ -28,6 +28,7 @@
 
 #if ENABLE(GPU_PROCESS) && USE(AUDIO_SESSION)
 
+#include "GPUConnectionToWebProcess.h"
 #include "GPUProcess.h"
 #include "GPUProcessConnectionMessages.h"
 #include "RemoteAudioSessionProxy.h"
@@ -35,6 +36,11 @@
 #include <WebCore/CoreAudioCaptureSource.h>
 #include <WebCore/PlatformMediaSessionManager.h>
 #include <wtf/HashCountedSet.h>
+#include <wtf/TZoneMallocInlines.h>
+
+#if PLATFORM(IOS_FAMILY)
+#include <WebCore/MediaSessionHelperIOS.h>
+#endif
 
 namespace WebKit {
 
@@ -45,17 +51,19 @@ static bool categoryCanMixWithOthers(AudioSession::CategoryType category)
     return category == AudioSession::CategoryType::AmbientSound;
 }
 
+WTF_MAKE_TZONE_ALLOCATED_IMPL(RemoteAudioSessionProxyManager);
+
 RemoteAudioSessionProxyManager::RemoteAudioSessionProxyManager(GPUProcess& gpuProcess)
     : m_gpuProcess(gpuProcess)
 {
-    AudioSession::sharedSession().addInterruptionObserver(*this);
-    AudioSession::sharedSession().addConfigurationChangeObserver(*this);
+    AudioSession::addInterruptionObserver(*this);
+    AudioSession::singleton().addConfigurationChangeObserver(*this);
 }
 
 RemoteAudioSessionProxyManager::~RemoteAudioSessionProxyManager()
 {
-    AudioSession::sharedSession().removeInterruptionObserver(*this);
-    AudioSession::sharedSession().removeConfigurationChangeObserver(*this);
+    AudioSession::removeInterruptionObserver(*this);
+    AudioSession::singleton().removeConfigurationChangeObserver(*this);
 }
 
 void RemoteAudioSessionProxyManager::addProxy(RemoteAudioSessionProxy& proxy, std::optional<audit_token_t> auditToken)
@@ -65,7 +73,7 @@ void RemoteAudioSessionProxyManager::addProxy(RemoteAudioSessionProxy& proxy, st
     updateCategory();
 
     if (auditToken)
-        AudioSession::sharedSession().setHostProcessAttribution(*auditToken);
+        AudioSession::singleton().setHostProcessAttribution(*auditToken);
 }
 
 void RemoteAudioSessionProxyManager::removeProxy(RemoteAudioSessionProxy& proxy)
@@ -80,10 +88,10 @@ void RemoteAudioSessionProxyManager::updateCategory()
     HashCountedSet<AudioSession::CategoryType, WTF::IntHash<AudioSession::CategoryType>, WTF::StrongEnumHashTraits<AudioSession::CategoryType>> categoryCounts;
     HashCountedSet<AudioSession::Mode, WTF::IntHash<AudioSession::Mode>, WTF::StrongEnumHashTraits<AudioSession::Mode>> modeCounts;
     HashCountedSet<RouteSharingPolicy, WTF::IntHash<RouteSharingPolicy>, WTF::StrongEnumHashTraits<RouteSharingPolicy>> policyCounts;
-    for (auto& otherProxy : m_proxies) {
-        categoryCounts.add(otherProxy.category());
-        modeCounts.add(otherProxy.mode());
-        policyCounts.add(otherProxy.routeSharingPolicy());
+    for (Ref otherProxy : m_proxies) {
+        categoryCounts.add(otherProxy->category());
+        modeCounts.add(otherProxy->mode());
+        policyCounts.add(otherProxy->routeSharingPolicy());
     }
 
     AudioSession::CategoryType category = AudioSession::CategoryType::None;
@@ -114,52 +122,43 @@ void RemoteAudioSessionProxyManager::updateCategory()
     else if (policyCounts.contains(RouteSharingPolicy::Independent))
         ASSERT_NOT_REACHED();
 
-    AudioSession::sharedSession().setCategory(category, mode, policy);
+    AudioSession::singleton().setCategory(category, mode, policy);
 }
 
 void RemoteAudioSessionProxyManager::updatePreferredBufferSizeForProcess()
 {
-#if ENABLE(MEDIA_STREAM)
-    if (CoreAudioCaptureSourceFactory::singleton().isAudioCaptureUnitRunning()) {
-        CoreAudioCaptureSourceFactory::singleton().whenAudioCaptureUnitIsNotRunning([weakThis = WeakPtr { *this }] {
-            if (weakThis)
-                weakThis->updatePreferredBufferSizeForProcess();
-        });
-        return;
-    }
-#endif
     size_t preferredBufferSize = std::numeric_limits<size_t>::max();
-    for (auto& proxy : m_proxies) {
-        if (proxy.preferredBufferSize() && proxy.preferredBufferSize() < preferredBufferSize)
-            preferredBufferSize = proxy.preferredBufferSize();
+    for (Ref proxy : m_proxies) {
+        if (proxy->preferredBufferSize() && proxy->preferredBufferSize() < preferredBufferSize)
+            preferredBufferSize = proxy->preferredBufferSize();
     }
 
     if (preferredBufferSize != std::numeric_limits<size_t>::max())
-        AudioSession::sharedSession().setPreferredBufferSize(preferredBufferSize);
+        AudioSession::singleton().setPreferredBufferSize(preferredBufferSize);
 }
 
 void RemoteAudioSessionProxyManager::updateSpatialExperience()
 {
     String sceneIdentifier;
     std::optional<AudioSession::SoundStageSize> maxSize;
-    for (auto& proxy : m_proxies) {
-        if (!proxy.isActive())
+    for (Ref proxy : m_proxies) {
+        if (!proxy->isActive())
             continue;
 
-        if (!maxSize || proxy.soundStageSize() > *maxSize) {
-            maxSize = proxy.soundStageSize();
-            sceneIdentifier = proxy.sceneIdentifier();
+        if (!maxSize || proxy->soundStageSize() > *maxSize) {
+            maxSize = proxy->soundStageSize();
+            sceneIdentifier = proxy->sceneIdentifier();
         }
     }
 
-    AudioSession::sharedSession().setSceneIdentifier(sceneIdentifier);
-    AudioSession::sharedSession().setSoundStageSize(maxSize.value_or(AudioSession::SoundStageSize::Automatic));
+    AudioSession::singleton().setSceneIdentifier(sceneIdentifier);
+    AudioSession::singleton().setSoundStageSize(maxSize.value_or(AudioSession::SoundStageSize::Automatic));
 }
 
 bool RemoteAudioSessionProxyManager::hasOtherActiveProxyThan(RemoteAudioSessionProxy& proxyToExclude)
 {
-    for (auto& proxy : m_proxies) {
-        if (proxy.isActive() && &proxy != &proxyToExclude)
+    for (Ref proxy : m_proxies) {
+        if (proxy->isActive() && proxy.ptr() != &proxyToExclude)
             return true;
     }
     return false;
@@ -167,13 +166,45 @@ bool RemoteAudioSessionProxyManager::hasOtherActiveProxyThan(RemoteAudioSessionP
 
 bool RemoteAudioSessionProxyManager::hasActiveNotInterruptedProxy()
 {
-    for (auto& proxy : m_proxies) {
-        if (proxy.isActive() && !proxy.isInterrupted())
+    for (Ref proxy : m_proxies) {
+        if (proxy->isActive() && !proxy->isInterrupted())
             return true;
     }
     return false;
 }
 
+#if PLATFORM(IOS_FAMILY)
+static void providePresentingApplicationPID(RemoteAudioSessionProxy& proxy)
+{
+    RefPtr webProcessConnection = proxy.gpuConnectionToWebProcess();
+    if (!webProcessConnection)
+        return;
+
+#if ENABLE(EXTENSION_CAPABILITIES)
+    if (webProcessConnection->sharedPreferencesForWebProcessValue().mediaCapabilityGrantsEnabled)
+        return;
+#endif
+
+    ProcessID pid = GPUProcess::singleton().parentProcessConnection()->remoteProcessID();
+
+#if !PLATFORM(APPLETV)
+    // Presenting application audit tokens are per-page, but AudioSessions are per-web-process,
+    // and it's not straightforward to know in the GPU process which page is responsible for activating
+    // the shared AVAudioSession. In reality, all pages in a given web process share a presenting
+    // application, so it's sufficient to use the first page's audit token.
+    auto& auditTokens = webProcessConnection->presentingApplicationAuditTokens();
+    if (!auditTokens.isEmpty())
+        pid = webProcessConnection->presentingApplicationPID(*auditTokens.keys().begin());
+#ifndef NDEBUG
+    for (auto& pageIdentifier : auditTokens.keys())
+        ASSERT(pid == webProcessConnection->presentingApplicationPID(pageIdentifier));
+#endif
+#endif
+
+    MediaSessionHelper::sharedHelper().providePresentingApplicationPID(pid);
+}
+#endif
+
 bool RemoteAudioSessionProxyManager::tryToSetActiveForProcess(RemoteAudioSessionProxy& proxy, bool active)
 {
     ASSERT(m_proxies.contains(proxy));
@@ -188,13 +219,17 @@ bool RemoteAudioSessionProxyManager::tryToSetActiveForProcess(RemoteAudioSession
         // This proxy wants to de-activate, and is the last remaining active
         // proxy. Deactivate the session, and return whether that deactivation
         // was sucessful.
-        return AudioSession::sharedSession().tryToSetActive(false);
+        return AudioSession::singleton().tryToSetActive(false);
     }
 
+#if PLATFORM(IOS_FAMILY)
+    providePresentingApplicationPID(proxy);
+#endif
+
     if (!hasActiveNotInterruptedProxy()) {
         // This proxy and only this proxy wants to become active. Activate
         // the session, and return whether that activation was successful.
-        return AudioSession::sharedSession().tryToSetActive(active);
+        return AudioSession::singleton().tryToSetActive(active);
     }
 
     // If this proxy is Ambient, and the session is already active, this
@@ -225,15 +260,8 @@ bool RemoteAudioSessionProxyManager::tryToSetActiveForProcess(RemoteAudioSession
 
 void RemoteAudioSessionProxyManager::updatePresentingProcesses()
 {
-#if ENABLE(EXTENSION_CAPABILITIES)
-    if (PlatformMediaSessionManager::mediaCapabilityGrantsEnabled())
-        return;
-#endif
-
     Vector<audit_token_t> presentingProcesses;
-
-    if (auto token = m_gpuProcess->parentProcessConnection()->getAuditToken())
-        presentingProcesses.append(*token);
+    bool shouldAppendParentProcess = false;
 
     // AVAudioSession will take out an assertion on all the "presenting applications"
     // when it moves to a "playing" state. But it's possible that (e.g.) multiple
@@ -241,30 +269,41 @@ void RemoteAudioSessionProxyManager::updatePresentingProcesses()
     // tokens from those proxies whose sessions are currently "active". Only their
     // presenting applications will be kept from becoming "suspended" during playback.
     m_proxies.forEach([&](auto& proxy) {
+        Ref gpuConnectionToWebProcess = *proxy.gpuConnectionToWebProcess();
+#if ENABLE(EXTENSION_CAPABILITIES)
+        if (gpuConnectionToWebProcess->sharedPreferencesForWebProcessValue().mediaCapabilityGrantsEnabled)
+            return;
+#endif
+        shouldAppendParentProcess = true;
         if (!proxy.isActive())
             return;
-        if (auto& token = proxy.gpuConnectionToWebProcess()->presentingApplicationAuditToken())
-            presentingProcesses.append(*token);
+        for (auto& token : gpuConnectionToWebProcess->presentingApplicationAuditTokens().values())
+            presentingProcesses.append(token.auditToken());
     });
-    AudioSession::sharedSession().setPresentingProcesses(WTFMove(presentingProcesses));
+
+    if (auto token = m_gpuProcess->protectedParentProcessConnection()->getAuditToken(); token && shouldAppendParentProcess)
+        presentingProcesses.append(*token);
+
+    if (!presentingProcesses.isEmpty())
+        AudioSession::singleton().setPresentingProcesses(WTFMove(presentingProcesses));
 }
 
 void RemoteAudioSessionProxyManager::beginInterruptionRemote()
 {
-    auto& session = this->session();
+    Ref session = this->session();
     // Temporarily remove as an observer to avoid a spurious IPC back to the web process.
-    session.removeInterruptionObserver(*this);
-    session.beginInterruption();
-    session.addInterruptionObserver(*this);
+    AudioSession::removeInterruptionObserver(*this);
+    session->beginInterruption();
+    AudioSession::addInterruptionObserver(*this);
 }
 
 void RemoteAudioSessionProxyManager::endInterruptionRemote(AudioSession::MayResume mayResume)
 {
-    auto& session = this->session();
+    Ref session = this->session();
     // Temporarily remove as an observer to avoid a spurious IPC back to the web process.
-    session.removeInterruptionObserver(*this);
-    session.endInterruption(mayResume);
-    session.addInterruptionObserver(*this);
+    AudioSession::removeInterruptionObserver(*this);
+    session->endInterruption(mayResume);
+    AudioSession::addInterruptionObserver(*this);
 }
 
 void RemoteAudioSessionProxyManager::beginAudioSessionInterruption()
